//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2002 Xilinx, Inc.
//     All rights reserved.
//

/*
-------------------------------------------------------------------------------
   Title      : Clockgen
   Project    : MicroBlaze Demo Board
-------------------------------------------------------------------------------
   File       : clockgen.v
   Company    : Xilinx, Inc.
   Created    : 2001/06/13
   Last Update: 2002/01/08
   Copyright  : (c) Xilinx Inc, 2001
-------------------------------------------------------------------------------
   Uses       : 
-------------------------------------------------------------------------------
   Used by    : Any user logic in the MicroBlaze demo board FPGA
-------------------------------------------------------------------------------
   Description: Clock generation for the MicroBlaze Multimedia Demo board FPGA

	This module is used to generate and distribute the various clocks on the 
	MicroBlaze demo board.

	The MASTER_CLOCK_P input is 27.00MHz and from this 54.00MHz and 108MHz 
	internal clocks are generated by the MAIN_DCM. A similar set of clocks 
	are created for the ZBT memories, by the MEMORY_DCM. This DCM uses an 
	external feedback loop to align the clock at the memory devices with 
	the internal clocks. 

	The ALTERNATE_CLOCK_P input is 50.00MHz and from this 40MHz is 
	generated by the PAL_PAL_NTSC_DCM for use as the NTSC pixel clock.

	A configuration dip switch determines if NTSC or PAL operation in required 
	and a BUFGMUX is used to select either the NTSC or PAL clock as the PIXEL clock 
	for the video DAC and the SVGA controller. 
	
	The EXTEND_DCM_RESET_P input is used to create a reset pulse to the DCMs when the
	FPGA is configured and "DONE" is asserted.

	
	Conventions:
		All port signals are UPPER CASE.
		Active HIGH PORT signals have a P suffex
		Active LOW PORT signals have a N suffex.
		All internal signals are LOWER CASE and are active HIGH.

-------------------------------------------------------------------------------
*/
module CLOCKGEN(
MASTER_CLOCK_P,
ALTERNATE_CLOCK_P,
MEM_CLK_FBOUT_P,
MEM_CLK_FBIN_P,
EXTEND_DCM_RESET_P,
clk_27mhz,
clk_54mhz,
clk_108mhz,
pixel_clock,
memory_27mhz,
memory_54mhz,
memory_108mhz,
pal_mode,
fpga_reset
);

input		MASTER_CLOCK_P;		// master clock input (27MHz) 
input		ALTERNATE_CLOCK_P;	// alternate clock input (50MHz)

output	MEM_CLK_FBOUT_P;		// memory DCM feedback loop output
input		MEM_CLK_FBIN_P;		// memory DCM feedback loop input
input		EXTEND_DCM_RESET_P;	// signal to extend the DCM reset
output	clk_27mhz;			// internal 27MHz clock
output	clk_54mhz;			// internal 54MHz clock
output	clk_108mhz;			// internal 108MHz clock
output	pixel_clock;		// pixel clock for the SVGA controller
output	memory_27mhz;		// 27MHz clock for the ZBT memories
output	memory_54mhz;		// 54MHz clock for the ZBT memories
output	memory_108mhz;		// 108MHz clock for the ZBT memories
input		pal_mode;			// selects PAL or NTSC clock as pixel_clock
output	fpga_reset;			// reset to fpga until all DCMs are locked

wire 		low;
wire		high;

wire		MASTER_CLOCK_P;
wire		ALTERNATE_CLOCK_P;
wire		MEM_CLK_FBOUT_P;
wire		MEM_CLK_FBIN_P;
wire 		EXTEND_DCM_RESET_P;

wire		clk_27mhz;
wire		clk_54mhz;
wire		clk_108mhz;
wire		clk27mhz;
wire		clk54mhz;
wire		clk108mhz;

wire		pixel_clock;
wire		memory_27mhz;
wire		memory_54mhz;
wire		memory_108mhz;
wire		pal_mode;
wire		fpga_reset;

wire		master_clock;
wire		alternate_clock;
wire		mem_clk_fbout;
wire		mem_clk_fbin;
wire		extend_dcm_reset;
wire		dcm_reset;

wire		main_locked;
wire 		memory_locked;
wire 		pal_ntsc_locked;
wire 		pal_clk;
wire 		ntsc_clk;

wire enable_reset_counter;

assign low 	= 1'b0;
assign high = 1'b1;

assign memory_27mhz = mem_clk_fbout;
//assign memory_27mhz = clk_27mhz;

reg [15:0] 	dcm_reset_counter;
reg 		terminal_count;


// reset the fpga until the DLLs are locked
assign fpga_reset = !(main_locked & memory_locked & pal_ntsc_locked);



//create a reset pulse for the DCM triggered by FPGA DONE
assign enable_reset_counter = !terminal_count & extend_dcm_reset;

initial
	begin
		dcm_reset_counter = 16'h0000;
	end

always @ (posedge master_clock) begin
	if (enable_reset_counter) begin
		dcm_reset_counter <= dcm_reset_counter +1;
		end
	end

always @ (dcm_reset_counter) begin
	if (dcm_reset_counter == 16'hFFFF)begin
		terminal_count = 1'b1;
		end
	else begin
		terminal_count = 1'b0;
		end
	end

assign dcm_reset = !dcm_reset_counter[15];


// instantiate the input buffers for the clocks and dcm reset
IBUFG MASTER_CLOCK (
.I(MASTER_CLOCK_P), 
.O(master_clock)
);

IBUFG ALTERNATE_CLOCK (
.I(ALTERNATE_CLOCK_P), 
.O(alternate_clock)
);

IBUF DCM_RESET (
.I(EXTEND_DCM_RESET_P), 
.O(extend_dcm_reset)
);

// instantiate the IO buffers for the memory feedback loop
IBUFG MEMORY_FB_IN (
.I(MEM_CLK_FBIN_P), 
.O(mem_clk_fbin)
);

OBUF_F_12 MEMORY_FB_OUT(
.O(MEM_CLK_FBOUT_P), 
.I(mem_clk_fbout)
);



// instantiate the global clock buffers

BUFG CLK27_BUF(
.O(clk_27mhz), 
.I(clk27mhz)
);

BUFG CLK54_BUF(
.O(clk_54mhz), 
.I(clk54mhz)
);

BUFG CLK108_BUF(
.O(clk_108mhz), 
.I(clk108mhz)
);

// instantiate the DCM for the internal clocks
DCM MAIN_DCM (
.CLKFB(clk_27mhz), 
.CLKIN(master_clock), 
.DSSEN(low), 
.PSCLK(low), 
.PSEN(low), 
.PSINCDEC(low), 
.RST(dcm_reset),
.CLK0(clk27mhz), 
.CLK90(), 
.CLK180(), 
.CLK270(), 
.CLK2X(clk54mhz), 
.CLK2X180(),
.CLKDV(), 
.CLKFX(clk108mhz), 
.CLKFX180(), 
.LOCKED(main_locked), 
.PSDONE(), 
.STATUS()
)
/* synthesis xc_props="DLL_FREQUENCY_MODE =LOW,DUTY_CYCLE_CORRECTION =TRUE,STARTUP_WAIT =FALSE,DFS_FREQUENCY_MODE =LOW,CLKFX_DIVIDE =1,CLKFX_MULTIPLY =4,CLK_FEEDBACK =1X,CLKOUT_PHASE_SHIFT =NONE,PHASE_SHIFT =0"*/;
/*
# define the DCMs
# fout = 27,54,108 MHz FOR INTERNAL LOGIC
inst  MAIN_DCM DLL_FREQUENCY_MODE = LOW;
inst  MAIN_DCM CLK_FEEDBACK = 1X;
inst  MAIN_DCM DUTY_CYCLE_CORRECTION = TRUE;
inst  MAIN_DCM STARTUP_WAIT = FALSE;
inst  MAIN_DCM DFS_FREQUENCY_MODE = LOW;
inst  MAIN_DCM CLKFX_DIVIDE = 1;
inst  MAIN_DCM CLKFX_MULTIPLY = 4;
inst  MAIN_DCM CLKOUT_PHASE_SHIFT = NONE;
inst  MAIN_DCM PHASE_SHIFT = 0;
*/

// instantiate the DCM for the memory clocks
DCM MEMORY_DCM (
.CLKFB(mem_clk_fbin), 
.CLKIN(master_clock), 
.DSSEN(low), 
.PSCLK(low), 
.PSEN(low), 
.PSINCDEC(low), 
.RST(dcm_reset),
.CLK0(mem_clk_fbout), 
.CLK90(), 
.CLK180(), 
.CLK270(), 
.CLK2X(memory_54mhz), 
.CLK2X180(),
.CLKDV(), 
.CLKFX(memory_108mhz), 
.CLKFX180(), 
.LOCKED(memory_locked), 
.PSDONE(), 
.STATUS()
)
/* synthesis xc_props="DLL_FREQUENCY_MODE =LOW,DUTY_CYCLE_CORRECTION =TRUE,STARTUP_WAIT =FALSE,DFS_FREQUENCY_MODE =LOW,CLKFX_DIVIDE =1,CLKFX_MULTIPLY =4,CLK_FEEDBACK =1X,CLKOUT_PHASE_SHIFT =NONE,PHASE_SHIFT =0"*/;
/*
# fout = 27,54,108 MHz FOR MEMORIES (uses external feedback loop)
inst  MEMORY_DCM DLL_FREQUENCY_MODE = LOW;
inst  MEMORY_DCM CLK_FEEDBACK = 1X;
inst  MEMORY_DCM DUTY_CYCLE_CORRECTION = TRUE;
inst  MEMORY_DCM STARTUP_WAIT = FALSE;
inst  MEMORY_DCM DFS_FREQUENCY_MODE = LOW;
inst  MEMORY_DCM CLKFX_DIVIDE = 1;
inst  MEMORY_DCM CLKFX_MULTIPLY = 4;
inst  MEMORY_DCM CLKOUT_PHASE_SHIFT = NONE;
inst  MEMORY_DCM PHASE_SHIFT = 0;
*/

// instantiate the DCM for the pixel clock
DCM PAL_NTSC_DCM (
.CLKFB(pal_clk), 
.CLKIN(alternate_clock), 
.DSSEN(low), 
.PSCLK(low), 
.PSEN(low), 
.PSINCDEC(low), 
.RST(dcm_reset),
.CLK0(pal_clk), 
.CLK90(), 
.CLK180(), 
.CLK270(), 
.CLK2X(), 
.CLK2X180(),
.CLKDV(), 
.CLKFX(ntsc_clk), 
.CLKFX180(), 
.LOCKED(pal_ntsc_locked), 
.PSDONE(), 
.STATUS()
)
/* synthesis xc_props="DLL_FREQUENCY_MODE =LOW,DUTY_CYCLE_CORRECTION =TRUE,STARTUP_WAIT =FALSE,DFS_FREQUENCY_MODE =LOW,CLKFX_DIVIDE =5,CLKFX_MULTIPLY =4,CLK_FEEDBACK =1X,CLKOUT_PHASE_SHIFT =NONE,PHASE_SHIFT =0"*/;
/*
# fout = 40.00MHz for 60Hz vertical refresh NTSC 
# fout = 50.00MHz for 75Hz vertical refresh PAL 
inst  PAL_NTSC_DCM DLL_FREQUENCY_MODE = LOW;
inst  PAL_NTSC_DCM CLK_FEEDBACK = 1X;
inst  PAL_NTSC_DCM DUTY_CYCLE_CORRECTION = TRUE;
inst  PAL_NTSC_DCM STARTUP_WAIT = FALSE;
inst  PAL_NTSC_DCM DFS_FREQUENCY_MODE = LOW;
inst  PAL_NTSC_DCM CLKFX_DIVIDE = 5;
inst  PAL_NTSC_DCM CLKFX_MULTIPLY = 4;
inst  PAL_NTSC_DCM CLKOUT_PHASE_SHIFT = NONE;
inst  PAL_NTSC_DCM PHASE_SHIFT = 0;
*/

//create the clock mux for the pixel clock
BUFGMUX PIXEL_CLK_MUX (.I0(ntsc_clk), .I1(pal_clk), .S(pal_mode), .O(pixel_clock)) /* synthesis syn_noclockbuf = 1 */;

// synthesis translate_off
defparam MAIN_DCM.DLL_FREQUENCY_MODE = "LOW";
defparam MAIN_DCM.DUTY_CYCLE_CORRECTION = "TRUE";
defparam MAIN_DCM.STARTUP_WAIT = "FALSE";
defparam MAIN_DCM.DFS_FREQUENCY_MODE = "LOW";
defparam MAIN_DCM.CLKFX_DIVIDE = 1;
defparam MAIN_DCM.CLKFX_MULTIPLY = 4;
defparam MAIN_DCM.CLK_FEEDBACK = "1X";
defparam MAIN_DCM.CLKOUT_PHASE_SHIFT = "NONE";
defparam MAIN_DCM.PHASE_SHIFT = 0;


defparam MEMORY_DCM.DLL_FREQUENCY_MODE = "LOW";
defparam MEMORY_DCM.DUTY_CYCLE_CORRECTION = "TRUE";
defparam MEMORY_DCM.STARTUP_WAIT = "FALSE";
defparam MEMORY_DCM.DFS_FREQUENCY_MODE = "LOW";
defparam MEMORY_DCM.CLKFX_DIVIDE = 1;
defparam MEMORY_DCM.CLKFX_MULTIPLY = 4;
defparam MEMORY_DCM.CLK_FEEDBACK = "1X";
defparam MEMORY_DCM.CLKOUT_PHASE_SHIFT = "NONE";
defparam MEMORY_DCM.PHASE_SHIFT = 0;

defparam PAL_NTSC_DCM.DLL_FREQUENCY_MODE = "LOW";
defparam PAL_NTSC_DCM.DUTY_CYCLE_CORRECTION = "TRUE";
defparam PAL_NTSC_DCM.STARTUP_WAIT = "FALSE";
defparam PAL_NTSC_DCM.DFS_FREQUENCY_MODE = "LOW";
defparam PAL_NTSC_DCM.CLKFX_DIVIDE = 5;
defparam PAL_NTSC_DCM.CLKFX_MULTIPLY = 4;
defparam PAL_NTSC_DCM.CLK_FEEDBACK = "1X";
defparam PAL_NTSC_DCM.CLKOUT_PHASE_SHIFT = "NONE";
defparam PAL_NTSC_DCM.PHASE_SHIFT = 0;

// synthesis translate_on

endmodule //CLOCKGEN

module BUFGMUX (O, I0, I1, S); // synthesis syn_black_box
input  I0;
input  I1;
input  S;
output O;
endmodule

module BUFG(O, I); // synthesis syn_black_box
output O;
input I;
endmodule

module IBUFG(O, I); // synthesis syn_black_box
output O;
input I;
endmodule

module IBUF(O, I); // synthesis syn_black_box
output O;
input I;
endmodule

module OBUF_F_12(O, I); // synthesis syn_black_box
output O;
input I;
endmodule

module DCM (CLKFB, CLKIN, DSSEN, PSCLK, PSEN, PSINCDEC, RST,
            CLK0, CLK90, CLK180, CLK270, CLK2X, CLK2X180,
            CLKDV, CLKFX, CLKFX180, LOCKED, PSDONE, STATUS); // synthesis syn_black_box

input CLKFB, CLKIN, DSSEN;
input PSCLK, PSEN, PSINCDEC, RST;

output CLK0, CLK90, CLK180, CLK270, CLK2X, CLK2X180;
output CLKDV, CLKFX, CLKFX180, LOCKED, PSDONE;
output [7:0] STATUS;
endmodule





